/*
 *  ZKMRNHALPlugInProtocol.mm
 *  Zirkonium
 *
 *  Created by Chandrasekhar Ramakrishnan on 28.02.07.
 *  Copyright 2007 C. Ramakrishnan/ZKM. All rights reserved.
 *
 */

#include "ZKMRNHALPlugInProtocol.h"
#include <unistd.h>

#pragma mark _____ ZirkoniumHALPort
CFDataRef ZirkoniumHALPort::PortManagerCallBack(CFMessagePortRef local, SInt32 msgid, CFDataRef data, void *info)
{
	ZirkoniumHALPort* manager = reinterpret_cast<ZirkoniumHALPort*>(info);
	return manager->ReceivedMessage(msgid, data);
}

ZirkoniumHALPort::ZirkoniumHALPort() : mMessagePort(NULL), mRunLoopSource(NULL), mData(NULL)
{	
	Boolean shouldFreeInfo;
	CFMessagePortContext context;
	context.info = this;
	context.version = 0; context.retain = NULL; context.release = NULL; context.copyDescription = NULL;	
	mMessagePort = CFMessagePortCreateLocal(NULL, NULL, PortManagerCallBack, &context, &shouldFreeInfo);
	if (shouldFreeInfo || (!mMessagePort)) {
		CFShow(CFSTR("Could not create message port"));
		mMessagePort = NULL;
		return;
	}
	
	mRunLoopSource = CFMessagePortCreateRunLoopSource(NULL, mMessagePort, 0);
	mData = CFDataCreateMutable(NULL, 0);
}

ZirkoniumHALPort::~ZirkoniumHALPort()
{
	CFRelease(mMessagePort);
	CFRelease(mRunLoopSource);
	CFRelease(mData);
}

void	ZirkoniumHALPort::ClearData()
{
	CFRange range = CFRangeMake(0, CFDataGetLength(mData));
	CFDataDeleteBytes(mData, range);
}

void	ZirkoniumHALPort::BeginMessage()
{
	ClearData();
	
	pid_t processID = getpid();
	CFDataAppendBytes(mData, (UInt8 *)&processID, sizeof(pid_t));
}

#pragma mark _____ ZirkoniumHALServerPort
ZirkoniumHALServerPort::ZirkoniumHALServerPort() : ZirkoniumHALPort() , mDelegate(NULL)
{ 
	CFMessagePortSetName(mMessagePort, CFSTR("ZirkoniumHALServerPort"));
}

ZirkoniumHALServerPort::~ZirkoniumHALServerPort()
{

}

#pragma mark _____ Heartbeating
SInt32		ZirkoniumHALServerPort::SendHeartbeatMessage(CFMessagePortRef port)
{
	ClearData();
	
	pid_t processID = getpid();
	CFDataAppendBytes(mData, (UInt8 *)&processID, sizeof(pid_t));
	
	return CFMessagePortSendRequest(port, kHeartbeatMessage, mData, 1., 1., NULL, NULL);	
}

#pragma mark _____ Device Setup
SInt32		ZirkoniumHALServerPort::SendOutputChannelMap(CFMessagePortRef port, UInt32 mapSize, SInt32* map)
{	
	ClearData();
	
	pid_t processID = getpid();
	CFDataAppendBytes(mData, (UInt8 *)&processID, sizeof(pid_t));
	
	CFDataAppendBytes(mData, (UInt8 *) &mapSize, sizeof(UInt32));
	CFDataAppendBytes(mData, (UInt8 *) map, sizeof(SInt32) * mapSize);
	return CFMessagePortSendRequest(port, kDeviceSetOutputChannelMapMessage, mData, 1., 1., NULL, NULL);
}

SInt32		ZirkoniumHALServerPort::SendSpeakerMode(CFMessagePortRef port, UInt8 numberOfInputs, UInt8 numberOfOutputs, UInt8 speakerMode, UInt8 simulationMode, CFDataRef speakerLayout)
{
	BeginMessage();
	
	CFDataAppendBytes(mData, (UInt8 *) &numberOfInputs, sizeof(UInt8));
	CFDataAppendBytes(mData, (UInt8 *) &numberOfOutputs, sizeof(UInt8));
	CFDataAppendBytes(mData, (UInt8 *) &speakerMode, sizeof(UInt8));
	CFDataAppendBytes(mData, (UInt8 *) &simulationMode, sizeof(UInt8));
	CFIndex lengthInBytes = CFDataGetLength(speakerLayout);
	CFDataAppendBytes(mData, (UInt8 *) &lengthInBytes, sizeof(CFIndex));
	CFDataAppendBytes(mData, CFDataGetBytePtr(speakerLayout), lengthInBytes);
	return CFMessagePortSendRequest(port, kDeviceSetSimulationMode, mData, 1., 1., NULL, NULL);
}

SInt32		ZirkoniumHALServerPort::SendSetNumberOfChannels(CFMessagePortRef port, UInt32 numberOfChannels)
{
	BeginMessage();
	
	CFDataAppendBytes(mData, (UInt8 *) &numberOfChannels, sizeof(UInt32));
	return CFMessagePortSendRequest(port, kDeviceSetNumberOfChannels, mData, 1., 1., NULL, NULL);
}

#pragma mark _____ Coeffs
SInt32		ZirkoniumHALServerPort::SendSetMatrix(CFMessagePortRef port, CFIndex lengthInBytes, Float32* coeffs)
{
	ClearData();
	
	pid_t processID = getpid();
	CFDataAppendBytes(mData, (UInt8 *)&processID, sizeof(pid_t));
	
	CFDataAppendBytes(mData, (UInt8 *) &lengthInBytes, sizeof(CFIndex));
	CFDataAppendBytes(mData, (UInt8 *) coeffs, lengthInBytes);
	return CFMessagePortSendRequest(port, kDeviceSetMatrixCoeffsMessage, mData, 1., 1., NULL, NULL);
}

#pragma mark _____ Debug
SInt32		ZirkoniumHALServerPort::SendLoggingLevel(CFMessagePortRef port, bool debugIsOn, UInt32 debugLevel)
{
	BeginMessage();
	
	CFDataAppendBytes(mData, (UInt8 *) &debugIsOn, sizeof(bool));
	CFDataAppendBytes(mData, (UInt8 *) &debugLevel, sizeof(UInt32));
	return CFMessagePortSendRequest(port, kDeviceSetLogLevel, mData, 1., 1., NULL, NULL);
}

#pragma mark _____ Internal Functions
CFDataRef	ZirkoniumHALServerPort::ReceivedMessage(SInt32 msgid, CFDataRef data)
{
	/// all messages begin with the pid
	CFRange range = { 0, sizeof(pid_t) };
	pid_t processID;
	CFDataGetBytes(data, range, (UInt8*) &processID);
	range.location += sizeof(pid_t); range.length = sizeof(CFIndex);

	switch(msgid) {
		case kCheckInMessage: 
		{
			// read the message
			CFIndex length; CFDataGetBytes(data, range, (UInt8*)&length);
			UInt8 bundleStr[256];
			range.location += sizeof(CFIndex); range.length = length;
			CFDataGetBytes(data, range, bundleStr);
			bundleStr[length] = '\0';
			if (mDelegate) mDelegate->CheckIn(processID, length, (char *) &bundleStr);
		} break;
		case kHeartbeatMessage:
			break;
		default:
			break;
	}
	return NULL;
}

#pragma mark _____ ZirkoniumHALClientPort
ZirkoniumHALClientPort::ZirkoniumHALClientPort() : ZirkoniumHALPort(), mRemoteMessagePort(NULL), mIsConnected(false), mIsLocalPortConnected(false)
{

}

ZirkoniumHALClientPort::~ZirkoniumHALClientPort()
{
	if (mRemoteMessagePort) CFRelease(mRemoteMessagePort);
}

#pragma mark _____ Actions
void		ZirkoniumHALClientPort::Connect()
{
	if (mIsConnected) return;
	
	if (!mIsLocalPortConnected) {
		CFStringRef clientPortName = CopyPortNameForPID(getpid());
		CFMessagePortSetName(mMessagePort, clientPortName);
		mIsLocalPortConnected = true;
		CFRelease(clientPortName);
	}
	mRemoteMessagePort = CFMessagePortCreateRemote(NULL, CFSTR("ZirkoniumHALServerPort"));
	if (!mRemoteMessagePort) {
		CFShow(CFSTR("Could not create remote message port"));
		mIsConnected = false;
		return;
	}
	mIsConnected = true;
	
	SendCheckInMessage();
}

#pragma mark _____ Internal Functions
CFDataRef	ZirkoniumHALClientPort::ReceivedMessage(SInt32 msgid, CFDataRef data)
{
	/// all messages begin with the pid
	CFRange range = { 0, sizeof(pid_t) };
	pid_t processID;
	CFDataGetBytes(data, range, (UInt8*) &processID);
	range.location += sizeof(pid_t);

	switch(msgid) {
		case kCheckInMessage:
			break;
		case kHeartbeatMessage:
			break;
		case kDeviceSetMatrixCoeffsMessage:
		{
			CFIndex lengthInBytes; Float32* coeffs;
			range.length = sizeof(CFIndex);
			CFDataGetBytes(data, range, (UInt8 *) &lengthInBytes);
			range.location += range.length; range.length = lengthInBytes;
			const UInt8 *dataPtr = CFDataGetBytePtr(data);
			coeffs = (Float32 *) (dataPtr + range.location);
			if (mDelegate) mDelegate->ReceiveSetMatrix(lengthInBytes, coeffs);
		} break;
		case kDeviceSetOutputChannelMapMessage:
		{
			UInt32 mapSize; SInt32* map;
			range.length = sizeof(UInt32);
			CFDataGetBytes(data, range, (UInt8 *) &mapSize);
			range.location += range.length; range.length = mapSize * sizeof(SInt32);
			const UInt8 *dataPtr = CFDataGetBytePtr(data);
			map = (SInt32 *) (dataPtr + range.location);
			if (mDelegate) mDelegate->ReceiveOutputChannelMap(mapSize, map);
		} break;
		case kDeviceSetSimulationMode:
		{
			UInt8 numberOfInputs;
			range.length = sizeof(UInt8);
			ReadAndAdvanceRange(data, range, &numberOfInputs);
			
			UInt8 numberOfOutputs;
			range.length = sizeof(UInt8);
			ReadAndAdvanceRange(data, range, &numberOfOutputs);
			
			UInt8 speakerMode;
			range.length = sizeof(UInt8);
			ReadAndAdvanceRange(data, range, &speakerMode);

			UInt8 simulationMode;
			range.length = sizeof(UInt8);
			ReadAndAdvanceRange(data, range, &simulationMode);

			CFIndex lengthInBytes;
			range.length = sizeof(CFIndex);	
			ReadAndAdvanceRange(data, range, &lengthInBytes);
			
			CFDataRef speakerLayout = 
				CFDataCreateWithBytesNoCopy(	NULL, 
												CFDataGetBytePtr(data) + range.location, 
												lengthInBytes, 
												kCFAllocatorNull);
			if (mDelegate) mDelegate->ReceiveSpeakerMode(numberOfInputs, numberOfOutputs, speakerMode, simulationMode, speakerLayout);
			CFRelease(speakerLayout);
		} break;
		case kDeviceSetNumberOfChannels:
		{
			UInt32 numberOfChannels;
			range.length = sizeof(UInt32);
			ReadAndAdvanceRange(data, range, &numberOfChannels);
			if (mDelegate) mDelegate->ReceiveNumberOfChannels(numberOfChannels);
		} break;
		case kDeviceSetLogLevel:
		{
			bool debugIsOn;
			range.length = sizeof(bool);
			ReadAndAdvanceRange(data, range, &debugIsOn);
			
			UInt32 debugLevel;
			range.length = sizeof(UInt32);
			ReadAndAdvanceRange(data, range, &debugLevel);
			
			if (mDelegate) mDelegate->ReceiveLogLevel(debugIsOn, debugLevel);
		}
		default:
			break;
	}
	return NULL;
}

void		ZirkoniumHALClientPort::SendCheckInMessage()
{
	pid_t processID = getpid();
	CFDataAppendBytes(mData, (UInt8 *)&processID, sizeof(pid_t));

	CFBundleRef mainBundle = CFBundleGetMainBundle();
	CFStringRef bundleID = CFBundleGetIdentifier(mainBundle);
	if (!bundleID) bundleID = CFSTR("Unknown");
	CFIndex length = (CFStringGetLength(bundleID)) < 256 ? CFStringGetLength(bundleID) : 256;
	UInt8 bundleStr[256];
	CFStringGetCString(bundleID, (char*) bundleStr, 256, kCFStringEncodingASCII);
	CFDataAppendBytes(mData, (UInt8*)&length, sizeof(CFIndex));
	CFDataAppendBytes(mData, bundleStr, length);
	SInt32 err = CFMessagePortSendRequest(mRemoteMessagePort, kCheckInMessage, mData, 1., 1., NULL, NULL);
	if (err != kCFMessagePortSuccess) printf("Failed to check in %i\n", err);
}

void		ZirkoniumHALClientPort::ReadAndAdvanceRange(CFDataRef data, CFRange& range, void* memoryLocation)
{
	CFDataGetBytes(data, range, (UInt8 *) memoryLocation);
 	range.location += range.length;
}
